"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from .basesdk import BaseSDK
from paygentic_sdk import errors, models, utils
from paygentic_sdk._hooks import HookContext
from paygentic_sdk.types import BaseModel, OptionalNullable, UNSET
from paygentic_sdk.utils import get_security_from_env
from paygentic_sdk.utils.unmarshal_json_response import unmarshal_json_response
from typing import Any, Mapping, Optional, Union, cast


class InvoicesV2(BaseSDK):
    r"""Invoice V2 operations supporting billing cycles organized by time periods. Warning: v0 invoice endpoints are no longer supported."""

    def list(
        self,
        *,
        request: Union[
            models.ListInvoicesRequest, models.ListInvoicesRequestTypedDict
        ] = models.ListInvoicesRequest(),
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.ListInvoicesResponse:
        r"""List

        List invoices with optional filters. Platform users can use nextActionAt=ready to get invoices ready for processing.

        :param request: The request object to send.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        if not isinstance(request, BaseModel):
            request = utils.unmarshal(request, models.ListInvoicesRequest)
        request = cast(models.ListInvoicesRequest, request)

        req = self._build_request(
            method="GET",
            path="/v2/invoices",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="listInvoices",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["403", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(models.ListInvoicesResponse, http_res)
        if utils.match_response(http_res, "403", "application/json"):
            response_data = unmarshal_json_response(errors.ErrorData, http_res)
            raise errors.Error(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(errors.ErrorData, http_res)
            raise errors.Error(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.PaygenticDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.PaygenticDefaultError(
                "API error occurred", http_res, http_res_text
            )

        raise errors.PaygenticDefaultError("Unexpected response received", http_res)

    async def list_async(
        self,
        *,
        request: Union[
            models.ListInvoicesRequest, models.ListInvoicesRequestTypedDict
        ] = models.ListInvoicesRequest(),
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.ListInvoicesResponse:
        r"""List

        List invoices with optional filters. Platform users can use nextActionAt=ready to get invoices ready for processing.

        :param request: The request object to send.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        if not isinstance(request, BaseModel):
            request = utils.unmarshal(request, models.ListInvoicesRequest)
        request = cast(models.ListInvoicesRequest, request)

        req = self._build_request_async(
            method="GET",
            path="/v2/invoices",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="listInvoices",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["403", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(models.ListInvoicesResponse, http_res)
        if utils.match_response(http_res, "403", "application/json"):
            response_data = unmarshal_json_response(errors.ErrorData, http_res)
            raise errors.Error(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(errors.ErrorData, http_res)
            raise errors.Error(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.PaygenticDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.PaygenticDefaultError(
                "API error occurred", http_res, http_res_text
            )

        raise errors.PaygenticDefaultError("Unexpected response received", http_res)

    def get(
        self,
        *,
        id: str,
        expand: Optional[str] = None,
        line_items_limit: Optional[int] = 100,
        line_items_page_token: Optional[str] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.Invoice:
        r"""Get

        Retrieve a single invoice with real-time aggregates (for ACTIVE/CLOSING/CLOSED) or cached aggregates (for finalized invoices). Optionally include line items with expand=lineItems.

        :param id: The invoice ID
        :param expand: Comma-separated list of fields to expand. Currently supports: lineItems
        :param line_items_limit: Page size for line items when expand=lineItems
        :param line_items_page_token: Pagination token for line items when expand=lineItems
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.GetInvoiceRequest(
            id=id,
            expand=expand,
            line_items_limit=line_items_limit,
            line_items_page_token=line_items_page_token,
        )

        req = self._build_request(
            method="GET",
            path="/v2/invoices/{id}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="getInvoice",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["403", "404", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(models.Invoice, http_res)
        if utils.match_response(http_res, ["403", "404"], "application/json"):
            response_data = unmarshal_json_response(errors.ErrorData, http_res)
            raise errors.Error(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(errors.ErrorData, http_res)
            raise errors.Error(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.PaygenticDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.PaygenticDefaultError(
                "API error occurred", http_res, http_res_text
            )

        raise errors.PaygenticDefaultError("Unexpected response received", http_res)

    async def get_async(
        self,
        *,
        id: str,
        expand: Optional[str] = None,
        line_items_limit: Optional[int] = 100,
        line_items_page_token: Optional[str] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.Invoice:
        r"""Get

        Retrieve a single invoice with real-time aggregates (for ACTIVE/CLOSING/CLOSED) or cached aggregates (for finalized invoices). Optionally include line items with expand=lineItems.

        :param id: The invoice ID
        :param expand: Comma-separated list of fields to expand. Currently supports: lineItems
        :param line_items_limit: Page size for line items when expand=lineItems
        :param line_items_page_token: Pagination token for line items when expand=lineItems
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.GetInvoiceRequest(
            id=id,
            expand=expand,
            line_items_limit=line_items_limit,
            line_items_page_token=line_items_page_token,
        )

        req = self._build_request_async(
            method="GET",
            path="/v2/invoices/{id}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="getInvoice",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["403", "404", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(models.Invoice, http_res)
        if utils.match_response(http_res, ["403", "404"], "application/json"):
            response_data = unmarshal_json_response(errors.ErrorData, http_res)
            raise errors.Error(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(errors.ErrorData, http_res)
            raise errors.Error(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.PaygenticDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.PaygenticDefaultError(
                "API error occurred", http_res, http_res_text
            )

        raise errors.PaygenticDefaultError("Unexpected response received", http_res)

    def get_line_items(
        self,
        *,
        id: str,
        limit: Optional[int] = 100,
        page_token: Optional[str] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.InvoiceLineItemsResponse:
        r"""Get Line Items

        Get paginated line items for an invoice from the analytics service

        :param id: The invoice ID
        :param limit: Maximum number of line items to return
        :param page_token: Token for pagination to fetch the next page of results
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.GetInvoiceLineItemsRequest(
            id=id,
            limit=limit,
            page_token=page_token,
        )

        req = self._build_request(
            method="GET",
            path="/v2/invoices/{id}/lineItems",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="getInvoiceLineItems",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["403", "404", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(models.InvoiceLineItemsResponse, http_res)
        if utils.match_response(http_res, ["403", "404"], "application/json"):
            response_data = unmarshal_json_response(errors.ErrorData, http_res)
            raise errors.Error(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(errors.ErrorData, http_res)
            raise errors.Error(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.PaygenticDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.PaygenticDefaultError(
                "API error occurred", http_res, http_res_text
            )

        raise errors.PaygenticDefaultError("Unexpected response received", http_res)

    async def get_line_items_async(
        self,
        *,
        id: str,
        limit: Optional[int] = 100,
        page_token: Optional[str] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.InvoiceLineItemsResponse:
        r"""Get Line Items

        Get paginated line items for an invoice from the analytics service

        :param id: The invoice ID
        :param limit: Maximum number of line items to return
        :param page_token: Token for pagination to fetch the next page of results
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.GetInvoiceLineItemsRequest(
            id=id,
            limit=limit,
            page_token=page_token,
        )

        req = self._build_request_async(
            method="GET",
            path="/v2/invoices/{id}/lineItems",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="getInvoiceLineItems",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["403", "404", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(models.InvoiceLineItemsResponse, http_res)
        if utils.match_response(http_res, ["403", "404"], "application/json"):
            response_data = unmarshal_json_response(errors.ErrorData, http_res)
            raise errors.Error(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(errors.ErrorData, http_res)
            raise errors.Error(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.PaygenticDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.PaygenticDefaultError(
                "API error occurred", http_res, http_res_text
            )

        raise errors.PaygenticDefaultError("Unexpected response received", http_res)



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from .paygenticerror import PaygenticError
from typing import Any, TYPE_CHECKING

from paygentic_sdk.utils.dynamic_imports import lazy_getattr, lazy_dir

if TYPE_CHECKING:
    from .badrequest import BadRequest, BadRequestUnion
    from .deletecustomerop import (
        DeleteCustomerConflictError,
        DeleteCustomerConflictErrorData,
    )
    from .deletefeeop import DeleteFeeConflictError, DeleteFeeConflictErrorData
    from .error import Error, ErrorData
    from .no_response_error import NoResponseError
    from .paygenticdefaulterror import PaygenticDefaultError
    from .responsevalidationerror import ResponseValidationError
    from .validationerror import ValidationError, ValidationErrorData

__all__ = [
    "BadRequest",
    "BadRequestUnion",
    "DeleteCustomerConflictError",
    "DeleteCustomerConflictErrorData",
    "DeleteFeeConflictError",
    "DeleteFeeConflictErrorData",
    "Error",
    "ErrorData",
    "NoResponseError",
    "PaygenticDefaultError",
    "PaygenticError",
    "ResponseValidationError",
    "ValidationError",
    "ValidationErrorData",
]

_dynamic_imports: dict[str, str] = {
    "BadRequest": ".badrequest",
    "BadRequestUnion": ".badrequest",
    "DeleteCustomerConflictError": ".deletecustomerop",
    "DeleteCustomerConflictErrorData": ".deletecustomerop",
    "DeleteFeeConflictError": ".deletefeeop",
    "DeleteFeeConflictErrorData": ".deletefeeop",
    "Error": ".error",
    "ErrorData": ".error",
    "NoResponseError": ".no_response_error",
    "PaygenticDefaultError": ".paygenticdefaulterror",
    "ResponseValidationError": ".responsevalidationerror",
    "ValidationError": ".validationerror",
    "ValidationErrorData": ".validationerror",
}


def __getattr__(attr_name: str) -> Any:
    return lazy_getattr(
        attr_name, package=__package__, dynamic_imports=_dynamic_imports
    )


def __dir__():
    return lazy_dir(dynamic_imports=_dynamic_imports)



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from dataclasses import dataclass, field
import httpx
from paygentic_sdk.errors import PaygenticError
from paygentic_sdk.models import validationerror as models_validationerror
from paygentic_sdk.types import BaseModel
from typing import List, Optional


class ValidationErrorData(BaseModel):
    message: str
    r"""Human-readable error message"""
    errors: List[models_validationerror.Error]
    r"""Array of field-specific validation errors"""
    error: Optional[models_validationerror.ErrorEnum] = "validation_error"
    r"""Error type indicating validation failure"""


@dataclass(unsafe_hash=True)
class ValidationError(PaygenticError):
    data: ValidationErrorData = field(hash=False)

    def __init__(
        self,
        data: ValidationErrorData,
        raw_response: httpx.Response,
        body: Optional[str] = None,
    ):
        fallback = body or raw_response.text
        message = str(data.message) or fallback
        super().__init__(message, raw_response, body)
        object.__setattr__(self, "data", data)



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from paygentic_sdk.types import BaseModel, UnrecognizedStr
import pydantic
from typing import Literal, Union
from typing_extensions import Annotated, TypedDict


EventType = Union[
    Literal[
        "usage",
        "fee",
    ],
    UnrecognizedStr,
]
r"""Type of event: 'usage' for billable metric events, 'fee' for fee events"""


LineItemType = Union[
    Literal[
        "charge",
        "refund",
    ],
    UnrecognizedStr,
]
r"""Type of line item: 'charge' for regular billing, 'refund' for refunded items (amounts are negated)"""


class InvoiceLineItemTypedDict(TypedDict):
    event_type: EventType
    r"""Type of event: 'usage' for billable metric events, 'fee' for fee events"""
    event_id: str
    r"""The event ID (usage_event_id or fee_event_id)"""
    event_source_id: str
    r"""Source ID: billable_metric_id for usage events, fee_id for fee events"""
    billable_metric_id: str
    r"""HOTFIX: Mirrors eventSourceId for backward compatibility"""
    invoice_display_name: str
    r"""Display name for this line item on invoices"""
    line_item_type: LineItemType
    r"""Type of line item: 'charge' for regular billing, 'refund' for refunded items (amounts are negated)"""
    meter_event_id: str
    r"""The meter event ID (usage events only)"""
    metric_description: str
    r"""Description of what this metric or fee measures"""
    metric_name: str
    r"""Human-readable name of the billable metric or fee"""
    metric_unit: str
    r"""Measurement unit label. Sample values: 'calls' for API calls, 'GB' for storage, 'hours' for compute time, 'tokens' for LLM tokens, 'charge' for fees"""
    quantity: float
    r"""Quantity of usage. Negative for refunds."""
    tax_rate: float
    r"""Tax rate as decimal (0.10 = 10%)"""
    total_amount: str
    r"""Total amount (totalPrice + totalTax) in decimal dollars. Negative for refunds."""
    total_price: str
    r"""Total price (tax-exclusive) in decimal dollars. Negative for refunds."""
    total_tax: str
    r"""Total tax in decimal dollars. Negative for refunds."""
    unit_price: str
    r"""Unit price in decimal dollars"""


class InvoiceLineItem(BaseModel):
    event_type: Annotated[EventType, pydantic.Field(alias="eventType")]
    r"""Type of event: 'usage' for billable metric events, 'fee' for fee events"""

    event_id: Annotated[str, pydantic.Field(alias="eventId")]
    r"""The event ID (usage_event_id or fee_event_id)"""

    event_source_id: Annotated[str, pydantic.Field(alias="eventSourceId")]
    r"""Source ID: billable_metric_id for usage events, fee_id for fee events"""

    billable_metric_id: Annotated[str, pydantic.Field(alias="billableMetricId")]
    r"""HOTFIX: Mirrors eventSourceId for backward compatibility"""

    invoice_display_name: Annotated[str, pydantic.Field(alias="invoiceDisplayName")]
    r"""Display name for this line item on invoices"""

    line_item_type: Annotated[LineItemType, pydantic.Field(alias="lineItemType")]
    r"""Type of line item: 'charge' for regular billing, 'refund' for refunded items (amounts are negated)"""

    meter_event_id: Annotated[str, pydantic.Field(alias="meterEventId")]
    r"""The meter event ID (usage events only)"""

    metric_description: Annotated[str, pydantic.Field(alias="metricDescription")]
    r"""Description of what this metric or fee measures"""

    metric_name: Annotated[str, pydantic.Field(alias="metricName")]
    r"""Human-readable name of the billable metric or fee"""

    metric_unit: Annotated[str, pydantic.Field(alias="metricUnit")]
    r"""Measurement unit label. Sample values: 'calls' for API calls, 'GB' for storage, 'hours' for compute time, 'tokens' for LLM tokens, 'charge' for fees"""

    quantity: float
    r"""Quantity of usage. Negative for refunds."""

    tax_rate: Annotated[float, pydantic.Field(alias="taxRate")]
    r"""Tax rate as decimal (0.10 = 10%)"""

    total_amount: Annotated[str, pydantic.Field(alias="totalAmount")]
    r"""Total amount (totalPrice + totalTax) in decimal dollars. Negative for refunds."""

    total_price: Annotated[str, pydantic.Field(alias="totalPrice")]
    r"""Total price (tax-exclusive) in decimal dollars. Negative for refunds."""

    total_tax: Annotated[str, pydantic.Field(alias="totalTax")]
    r"""Total tax in decimal dollars. Negative for refunds."""

    unit_price: Annotated[str, pydantic.Field(alias="unitPrice")]
    r"""Unit price in decimal dollars"""


try:
    InvoiceLineItem.model_rebuild()
except NameError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from datetime import datetime
from decimal import Decimal
from paygentic_sdk.types import (
    BaseModel,
    Nullable,
    OptionalNullable,
    UNSET,
    UNSET_SENTINEL,
    UnrecognizedStr,
)
from paygentic_sdk.utils import serialize_decimal, validate_decimal
import pydantic
from pydantic import model_serializer
from pydantic.functional_serializers import PlainSerializer
from pydantic.functional_validators import BeforeValidator
from typing import Literal, Optional, Union
from typing_extensions import Annotated, NotRequired, TypedDict


DisputeObject = Literal["dispute",]


DisputeStatus = Union[
    Literal[
        "pending",
        "accepted",
        "declined",
    ],
    UnrecognizedStr,
]
r"""Current status of the dispute"""


class DisputeTypedDict(TypedDict):
    id: str
    r"""Unique identifier for a dispute"""
    created_at: datetime
    customer_id: str
    r"""Customer who filed the dispute"""
    customer_message: str
    r"""Customer's reason for the dispute"""
    disputed_amount: Decimal
    r"""Disputed charge amount in decimal dollar format. Sample values: '2.50' equals $2.50 disputed, '15.00' equals $15.00 disputed, '0.50' equals $0.50 disputed"""
    merchant_id: str
    r"""Unique identifier for an organization"""
    status: DisputeStatus
    r"""Current status of the dispute"""
    updated_at: datetime
    usage_event_id: str
    r"""The usage event being disputed"""
    object: NotRequired[DisputeObject]
    merchant_message: NotRequired[Nullable[str]]
    r"""Merchant's response to the dispute"""
    resolved_at: NotRequired[Nullable[datetime]]
    r"""When the dispute was resolved"""


class Dispute(BaseModel):
    id: str
    r"""Unique identifier for a dispute"""

    created_at: Annotated[datetime, pydantic.Field(alias="createdAt")]

    customer_id: Annotated[str, pydantic.Field(alias="customerId")]
    r"""Customer who filed the dispute"""

    customer_message: Annotated[str, pydantic.Field(alias="customerMessage")]
    r"""Customer's reason for the dispute"""

    disputed_amount: Annotated[
        Annotated[
            Decimal,
            BeforeValidator(validate_decimal),
            PlainSerializer(serialize_decimal(True)),
        ],
        pydantic.Field(alias="disputedAmount"),
    ]
    r"""Disputed charge amount in decimal dollar format. Sample values: '2.50' equals $2.50 disputed, '15.00' equals $15.00 disputed, '0.50' equals $0.50 disputed"""

    merchant_id: Annotated[str, pydantic.Field(alias="merchantId")]
    r"""Unique identifier for an organization"""

    status: DisputeStatus
    r"""Current status of the dispute"""

    updated_at: Annotated[datetime, pydantic.Field(alias="updatedAt")]

    usage_event_id: Annotated[str, pydantic.Field(alias="usageEventId")]
    r"""The usage event being disputed"""

    object: Optional[DisputeObject] = "dispute"

    merchant_message: Annotated[
        OptionalNullable[str], pydantic.Field(alias="merchantMessage")
    ] = UNSET
    r"""Merchant's response to the dispute"""

    resolved_at: Annotated[
        OptionalNullable[datetime], pydantic.Field(alias="resolvedAt")
    ] = UNSET
    r"""When the dispute was resolved"""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["object", "merchantMessage", "resolvedAt"])
        nullable_fields = set(["merchantMessage", "resolvedAt"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            is_nullable_and_explicitly_set = (
                k in nullable_fields
                and (self.__pydantic_fields_set__.intersection({n}))  # pylint: disable=no-member
            )

            if val != UNSET_SENTINEL:
                if (
                    val is not None
                    or k not in optional_fields
                    or is_nullable_and_explicitly_set
                ):
                    m[k] = val

        return m


try:
    Dispute.model_rebuild()
except NameError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .pricefeatureinput import PriceFeatureInput, PriceFeatureInputTypedDict
from .priceproperties_union import PricePropertiesUnion, PricePropertiesUnionTypedDict
from paygentic_sdk.types import (
    BaseModel,
    Nullable,
    OptionalNullable,
    UNSET,
    UNSET_SENTINEL,
)
import pydantic
from pydantic import model_serializer
from typing import Literal, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


CreatePriceModel = Literal[
    "standard",
    "dynamic",
    "volume",
    "percentage",
]
r"""Pricing calculation model. Required for billable metrics, optional for fees (defaults to 'standard')."""


CreatePricePaymentTerm = Literal[
    "instant",
    "in_arrears",
    "in_advance",
]
r"""Billing timing preference. For billable metrics: 'instant' (charges immediately) or 'in_arrears' (charges at period end). For fees: 'in_advance' (charges upfront) or 'in_arrears' (charges at period end)."""


class CreatePriceRequestTypedDict(TypedDict):
    invoice_display_name: str
    r"""Line item label shown on customer invoices. Sample values: 'Claude Token Consumption', 'Storage Usage (GB)', 'Inference API Calls', 'Image Generation Count', 'Training Compute Hours', 'Data Transfer (TB)'"""
    payment_term: CreatePricePaymentTerm
    r"""Billing timing preference. For billable metrics: 'instant' (charges immediately) or 'in_arrears' (charges at period end). For fees: 'in_advance' (charges upfront) or 'in_arrears' (charges at period end)."""
    properties: PricePropertiesUnionTypedDict
    billable_metric_id: NotRequired[str]
    r"""Unique identifier for a billable metric"""
    fee_id: NotRequired[str]
    r"""The unique identifier for the fee referred to by this price. Either billableMetricId or feeId must be provided."""
    model: NotRequired[CreatePriceModel]
    r"""Pricing calculation model. Required for billable metrics, optional for fees (defaults to 'standard')."""
    billing_cadence: NotRequired[Nullable[str]]
    r"""ISO 8601 duration for recurring charges (e.g., 'P1M' for monthly, 'P1Y' for yearly) or 'P0D' for one-time charges. Required for fees, optional for billable metrics. Sample values: 'P0D' for one-time, 'P1M' for monthly recurring, 'P1Y' for yearly recurring"""
    feature: NotRequired[PriceFeatureInputTypedDict]


class CreatePriceRequest(BaseModel):
    invoice_display_name: Annotated[str, pydantic.Field(alias="invoiceDisplayName")]
    r"""Line item label shown on customer invoices. Sample values: 'Claude Token Consumption', 'Storage Usage (GB)', 'Inference API Calls', 'Image Generation Count', 'Training Compute Hours', 'Data Transfer (TB)'"""

    payment_term: Annotated[CreatePricePaymentTerm, pydantic.Field(alias="paymentTerm")]
    r"""Billing timing preference. For billable metrics: 'instant' (charges immediately) or 'in_arrears' (charges at period end). For fees: 'in_advance' (charges upfront) or 'in_arrears' (charges at period end)."""

    properties: PricePropertiesUnion

    billable_metric_id: Annotated[
        Optional[str], pydantic.Field(alias="billableMetricId")
    ] = None
    r"""Unique identifier for a billable metric"""

    fee_id: Annotated[Optional[str], pydantic.Field(alias="feeId")] = None
    r"""The unique identifier for the fee referred to by this price. Either billableMetricId or feeId must be provided."""

    model: Optional[CreatePriceModel] = None
    r"""Pricing calculation model. Required for billable metrics, optional for fees (defaults to 'standard')."""

    billing_cadence: Annotated[
        OptionalNullable[str], pydantic.Field(alias="billingCadence")
    ] = UNSET
    r"""ISO 8601 duration for recurring charges (e.g., 'P1M' for monthly, 'P1Y' for yearly) or 'P0D' for one-time charges. Required for fees, optional for billable metrics. Sample values: 'P0D' for one-time, 'P1M' for monthly recurring, 'P1Y' for yearly recurring"""

    feature: Optional[PriceFeatureInput] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(
            ["billableMetricId", "feeId", "model", "billingCadence", "feature"]
        )
        nullable_fields = set(["billingCadence"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            is_nullable_and_explicitly_set = (
                k in nullable_fields
                and (self.__pydantic_fields_set__.intersection({n}))  # pylint: disable=no-member
            )

            if val != UNSET_SENTINEL:
                if (
                    val is not None
                    or k not in optional_fields
                    or is_nullable_and_explicitly_set
                ):
                    m[k] = val

        return m


try:
    CreatePriceRequest.model_rebuild()
except NameError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .rulecondition import RuleCondition, RuleConditionTypedDict
from paygentic_sdk.types import BaseModel, UNSET_SENTINEL
from pydantic import model_serializer
from typing import List, Optional
from typing_extensions import NotRequired, TypedDict


class UpdateRuleRequestTypedDict(TypedDict):
    conditions: NotRequired[List[RuleConditionTypedDict]]
    r"""List of conditions that must ALL be met (AND logic)"""
    description: NotRequired[str]
    r"""Detailed description of what this source rule does"""
    enabled: NotRequired[bool]
    r"""Whether this rule should be active"""
    name: NotRequired[str]
    r"""Human-readable name for the source rule"""
    order: NotRequired[int]
    r"""Priority order for rule evaluation (lower numbers are evaluated first)"""


class UpdateRuleRequest(BaseModel):
    conditions: Optional[List[RuleCondition]] = None
    r"""List of conditions that must ALL be met (AND logic)"""

    description: Optional[str] = None
    r"""Detailed description of what this source rule does"""

    enabled: Optional[bool] = None
    r"""Whether this rule should be active"""

    name: Optional[str] = None
    r"""Human-readable name for the source rule"""

    order: Optional[int] = None
    r"""Priority order for rule evaluation (lower numbers are evaluated first)"""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["conditions", "description", "enabled", "name", "order"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from paygentic_sdk.types import BaseModel, UNSET_SENTINEL
import pydantic
from pydantic import model_serializer
from typing import Any, Dict, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class PriceFeatureInputTypedDict(TypedDict):
    feature_id: str
    r"""The feature to associate with this price"""
    entitlement_template: NotRequired[Dict[str, Any]]
    r"""Template for entitlement values when this feature is provisioned"""


class PriceFeatureInput(BaseModel):
    feature_id: Annotated[str, pydantic.Field(alias="featureId")]
    r"""The feature to associate with this price"""

    entitlement_template: Annotated[
        Optional[Dict[str, Any]], pydantic.Field(alias="entitlementTemplate")
    ] = None
    r"""Template for entitlement values when this feature is provisioned"""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["entitlementTemplate"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


try:
    PriceFeatureInput.model_rebuild()
except NameError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .offsetpagination import OffsetPagination, OffsetPaginationTypedDict
from .subscription import Subscription, SubscriptionTypedDict
from paygentic_sdk.types import BaseModel, UNSET_SENTINEL
from paygentic_sdk.utils import FieldMetadata, QueryParamMetadata
import pydantic
from pydantic import model_serializer
from typing import List, Literal, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


ListSubscriptionsStatus = Literal[
    "active",
    "terminated",
]


class ListSubscriptionsRequestTypedDict(TypedDict):
    consumer_id: NotRequired[str]
    customer_id: NotRequired[str]
    limit: NotRequired[str]
    r"""Number of subscriptions to return"""
    merchant_id: NotRequired[str]
    offset: NotRequired[str]
    r"""Number of subscriptions to skip"""
    status: NotRequired[ListSubscriptionsStatus]


class ListSubscriptionsRequest(BaseModel):
    consumer_id: Annotated[
        Optional[str],
        pydantic.Field(alias="consumerId"),
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = None

    customer_id: Annotated[
        Optional[str],
        pydantic.Field(alias="customerId"),
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = None

    limit: Annotated[
        Optional[str],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = "10"
    r"""Number of subscriptions to return"""

    merchant_id: Annotated[
        Optional[str],
        pydantic.Field(alias="merchantId"),
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = None

    offset: Annotated[
        Optional[str],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = "0"
    r"""Number of subscriptions to skip"""

    status: Annotated[
        Optional[ListSubscriptionsStatus],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(
            ["consumerId", "customerId", "limit", "merchantId", "offset", "status"]
        )
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


class ListSubscriptionsResponseTypedDict(TypedDict):
    r"""List of subscriptions"""

    data: NotRequired[List[SubscriptionTypedDict]]
    pagination: NotRequired[OffsetPaginationTypedDict]
    r"""Offset-based pagination response."""


class ListSubscriptionsResponse(BaseModel):
    r"""List of subscriptions"""

    data: Optional[List[Subscription]] = None

    pagination: Optional[OffsetPagination] = None
    r"""Offset-based pagination response."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["data", "pagination"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .address import Address, AddressTypedDict
from .validtaxaddress import ValidTaxAddress, ValidTaxAddressTypedDict
from paygentic_sdk.types import BaseModel, UNSET_SENTINEL
import pydantic
from pydantic import model_serializer
from typing import Dict, Optional, Union
from typing_extensions import Annotated, NotRequired, TypeAliasType, TypedDict


class ConsumerTypedDict(TypedDict):
    r"""Fields to create a new consumer. Will use an existing consumer if one exists with the same email address. Required if `consumerId` is not provided. Address with complete tax information (country, state, zipCode) is required for tax calculation when using Paygentic Tax."""

    name: str
    email: str
    r"""Consumer email address. Sample values: 'customer@example.com', 'billing@company.com', 'admin@startup.io'"""
    address: AddressTypedDict
    r"""Consumer address. For accurate tax calculation when using Paygentic Tax, provide complete address with country (ISO 3166-1 alpha-2), state, and zipCode. Incomplete addresses will skip tax calculation and use the plan's default tax rate instead."""
    phone: NotRequired[str]
    r"""Consumer phone number. Sample values: '+1-555-123-4567', '+44-20-7946-0958', '+49-30-12345678'"""


class Consumer(BaseModel):
    r"""Fields to create a new consumer. Will use an existing consumer if one exists with the same email address. Required if `consumerId` is not provided. Address with complete tax information (country, state, zipCode) is required for tax calculation when using Paygentic Tax."""

    name: str

    email: str
    r"""Consumer email address. Sample values: 'customer@example.com', 'billing@company.com', 'admin@startup.io'"""

    address: Address
    r"""Consumer address. For accurate tax calculation when using Paygentic Tax, provide complete address with country (ISO 3166-1 alpha-2), state, and zipCode. Incomplete addresses will skip tax calculation and use the plan's default tax rate instead."""

    phone: Optional[str] = None
    r"""Consumer phone number. Sample values: '+1-555-123-4567', '+44-20-7946-0958', '+49-30-12345678'"""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["phone"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


class CreateCustomerRequestTypedDict(TypedDict):
    merchant_id: str
    r"""Unique identifier for an organization"""
    consumer: NotRequired[ConsumerTypedDict]
    r"""Fields to create a new consumer. Will use an existing consumer if one exists with the same email address. Required if `consumerId` is not provided. Address with complete tax information (country, state, zipCode) is required for tax calculation when using Paygentic Tax."""
    consumer_id: NotRequired[str]
    r"""Unique identifier for an organization"""
    tax_id: NotRequired[str]
    r"""Optional business tax registration identifier. Sample values: 'GB123456789' for UK VAT, 'DE123456789' for German VAT, 'FR12345678901' for French VAT. Supplying this value enables inter-company tax handling and exemption from standard tax collection."""
    tax_rates: NotRequired[Dict[str, float]]
    r"""An object mapping plan IDs, metric IDs, or 'default' to a tax rate percentage (e.g., 13 for 13%)"""


class CreateCustomerRequest(BaseModel):
    merchant_id: Annotated[str, pydantic.Field(alias="merchantId")]
    r"""Unique identifier for an organization"""

    consumer: Optional[Consumer] = None
    r"""Fields to create a new consumer. Will use an existing consumer if one exists with the same email address. Required if `consumerId` is not provided. Address with complete tax information (country, state, zipCode) is required for tax calculation when using Paygentic Tax."""

    consumer_id: Annotated[Optional[str], pydantic.Field(alias="consumerId")] = None
    r"""Unique identifier for an organization"""

    tax_id: Annotated[Optional[str], pydantic.Field(alias="taxId")] = None
    r"""Optional business tax registration identifier. Sample values: 'GB123456789' for UK VAT, 'DE123456789' for German VAT, 'FR12345678901' for French VAT. Supplying this value enables inter-company tax handling and exemption from standard tax collection."""

    tax_rates: Annotated[
        Optional[Dict[str, float]], pydantic.Field(alias="taxRates")
    ] = None
    r"""An object mapping plan IDs, metric IDs, or 'default' to a tax rate percentage (e.g., 13 for 13%)"""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["consumer", "consumerId", "taxId", "taxRates"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


class CreateCustomerResponseBody2TypedDict(TypedDict):
    r"""Customer created successfully"""

    customer_id: str
    valid_tax_address: ValidTaxAddressTypedDict
    r"""Indicates whether the consumer address is valid for tax calculation when using Paygentic Tax. If valid=false, tax calculation will be skipped and internal invoice flow with default tax rate will be used."""


class CreateCustomerResponseBody2(BaseModel):
    r"""Customer created successfully"""

    customer_id: Annotated[str, pydantic.Field(alias="customerId")]

    valid_tax_address: Annotated[
        ValidTaxAddress, pydantic.Field(alias="validTaxAddress")
    ]
    r"""Indicates whether the consumer address is valid for tax calculation when using Paygentic Tax. If valid=false, tax calculation will be skipped and internal invoice flow with default tax rate will be used."""


class CreateCustomerResponseBody1TypedDict(TypedDict):
    r"""Customer already exists"""

    customer_id: str
    valid_tax_address: ValidTaxAddressTypedDict
    r"""Indicates whether the consumer address is valid for tax calculation when using Paygentic Tax. If valid=false, tax calculation will be skipped and internal invoice flow with default tax rate will be used."""


class CreateCustomerResponseBody1(BaseModel):
    r"""Customer already exists"""

    customer_id: Annotated[str, pydantic.Field(alias="customerId")]

    valid_tax_address: Annotated[
        ValidTaxAddress, pydantic.Field(alias="validTaxAddress")
    ]
    r"""Indicates whether the consumer address is valid for tax calculation when using Paygentic Tax. If valid=false, tax calculation will be skipped and internal invoice flow with default tax rate will be used."""


CreateCustomerResponseTypedDict = TypeAliasType(
    "CreateCustomerResponseTypedDict",
    Union[CreateCustomerResponseBody1TypedDict, CreateCustomerResponseBody2TypedDict],
)


CreateCustomerResponse = TypeAliasType(
    "CreateCustomerResponse",
    Union[CreateCustomerResponseBody1, CreateCustomerResponseBody2],
)


try:
    CreateCustomerRequest.model_rebuild()
except NameError:
    pass
try:
    CreateCustomerResponseBody2.model_rebuild()
except NameError:
    pass
try:
    CreateCustomerResponseBody1.model_rebuild()
except NameError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from paygentic_sdk.types import BaseModel, UNSET_SENTINEL
from paygentic_sdk.utils import FieldMetadata, PathParamMetadata, RequestMetadata
import pydantic
from pydantic import model_serializer
from typing import List, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class BulkApproveSourceEventsRequestBodyTypedDict(TypedDict):
    event_ids: List[str]
    r"""IDs of events to approve"""


class BulkApproveSourceEventsRequestBody(BaseModel):
    event_ids: Annotated[List[str], pydantic.Field(alias="eventIds")]
    r"""IDs of events to approve"""


class BulkApproveSourceEventsRequestTypedDict(TypedDict):
    id: str
    request_body: BulkApproveSourceEventsRequestBodyTypedDict


class BulkApproveSourceEventsRequest(BaseModel):
    id: Annotated[
        str, FieldMetadata(path=PathParamMetadata(style="simple", explode=False))
    ]

    request_body: Annotated[
        BulkApproveSourceEventsRequestBody,
        FieldMetadata(request=RequestMetadata(media_type="application/json")),
    ]


class BulkApproveSourceEventsFailedTypedDict(TypedDict):
    id: NotRequired[str]
    error: NotRequired[str]


class BulkApproveSourceEventsFailed(BaseModel):
    id: Optional[str] = None

    error: Optional[str] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["id", "error"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


class BulkApproveSourceEventsDetailsTypedDict(TypedDict):
    processed: NotRequired[List[str]]
    r"""IDs of successfully processed events"""
    failed: NotRequired[List[BulkApproveSourceEventsFailedTypedDict]]
    r"""Failed events with error messages"""


class BulkApproveSourceEventsDetails(BaseModel):
    processed: Optional[List[str]] = None
    r"""IDs of successfully processed events"""

    failed: Optional[List[BulkApproveSourceEventsFailed]] = None
    r"""Failed events with error messages"""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["processed", "failed"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


class BulkApproveSourceEventsResponseTypedDict(TypedDict):
    r"""Bulk approval results"""

    processed: NotRequired[int]
    r"""Number of successfully approved events"""
    failed: NotRequired[int]
    r"""Number of events that failed to process"""
    details: NotRequired[BulkApproveSourceEventsDetailsTypedDict]


class BulkApproveSourceEventsResponse(BaseModel):
    r"""Bulk approval results"""

    processed: Optional[int] = None
    r"""Number of successfully approved events"""

    failed: Optional[int] = None
    r"""Number of events that failed to process"""

    details: Optional[BulkApproveSourceEventsDetails] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["processed", "failed", "details"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


try:
    BulkApproveSourceEventsRequestBody.model_rebuild()
except NameError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from datetime import datetime
from paygentic_sdk.types import BaseModel, UNSET_SENTINEL, UnrecognizedStr
import pydantic
from pydantic import model_serializer
from typing import Dict, List, Literal, Optional, Union
from typing_extensions import Annotated, NotRequired, TypedDict


PaymentObject = Literal["payment",]


class LineItemTypedDict(TypedDict):
    description: NotRequired[str]
    amount: NotRequired[str]
    quantity: NotRequired[int]


class LineItem(BaseModel):
    description: Optional[str] = None

    amount: Optional[str] = None

    quantity: Optional[int] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["description", "amount", "quantity"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


PaymentStatus = Union[
    Literal[
        "pending",
        "processing",
        "completed",
        "expired",
        "cancelled",
    ],
    UnrecognizedStr,
]
r"""Current status of the payment."""


class PaymentTypedDict(TypedDict):
    id: str
    r"""Unique payment identifier (pay_* prefixed)."""
    amount: str
    r"""Payment amount in decimal format (e.g. '10.50')."""
    currency: str
    r"""ISO 4217 currency code (e.g. 'USD')."""
    status: PaymentStatus
    r"""Current status of the payment."""
    created_at: datetime
    r"""When the payment was created."""
    object: NotRequired[PaymentObject]
    payment_url: NotRequired[str]
    r"""URL for the customer to complete the payment."""
    customer_id: NotRequired[str]
    r"""Customer ID if provided."""
    idempotency_key: NotRequired[str]
    r"""Client-provided idempotency key."""
    reference: NotRequired[str]
    r"""Merchant-defined reference for this payment."""
    metadata: NotRequired[Dict[str, str]]
    r"""Merchant-provided key-value metadata."""
    line_items: NotRequired[List[LineItemTypedDict]]
    r"""Breakdown of what the customer is being charged for."""
    expires_at: NotRequired[datetime]
    r"""When the payment expires."""


class Payment(BaseModel):
    id: str
    r"""Unique payment identifier (pay_* prefixed)."""

    amount: str
    r"""Payment amount in decimal format (e.g. '10.50')."""

    currency: str
    r"""ISO 4217 currency code (e.g. 'USD')."""

    status: PaymentStatus
    r"""Current status of the payment."""

    created_at: Annotated[datetime, pydantic.Field(alias="createdAt")]
    r"""When the payment was created."""

    object: Optional[PaymentObject] = "payment"

    payment_url: Annotated[Optional[str], pydantic.Field(alias="paymentUrl")] = None
    r"""URL for the customer to complete the payment."""

    customer_id: Annotated[Optional[str], pydantic.Field(alias="customerId")] = None
    r"""Customer ID if provided."""

    idempotency_key: Annotated[
        Optional[str], pydantic.Field(alias="idempotencyKey")
    ] = None
    r"""Client-provided idempotency key."""

    reference: Optional[str] = None
    r"""Merchant-defined reference for this payment."""

    metadata: Optional[Dict[str, str]] = None
    r"""Merchant-provided key-value metadata."""

    line_items: Annotated[
        Optional[List[LineItem]], pydantic.Field(alias="lineItems")
    ] = None
    r"""Breakdown of what the customer is being charged for."""

    expires_at: Annotated[Optional[datetime], pydantic.Field(alias="expiresAt")] = None
    r"""When the payment expires."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(
            [
                "object",
                "paymentUrl",
                "customerId",
                "idempotencyKey",
                "reference",
                "metadata",
                "lineItems",
                "expiresAt",
            ]
        )
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


try:
    Payment.model_rebuild()
except NameError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .offsetpagination import OffsetPagination, OffsetPaginationTypedDict
from .product import Product, ProductTypedDict
from paygentic_sdk.types import BaseModel, UNSET_SENTINEL
from paygentic_sdk.utils import FieldMetadata, QueryParamMetadata
import pydantic
from pydantic import model_serializer
from typing import List, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class ListProductsRequestTypedDict(TypedDict):
    limit: NotRequired[int]
    r"""Number of products to return"""
    merchant_id: NotRequired[str]
    r"""Filter products by merchant organization ID"""
    offset: NotRequired[int]
    r"""Number of products to skip"""


class ListProductsRequest(BaseModel):
    limit: Annotated[
        Optional[int],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = 10
    r"""Number of products to return"""

    merchant_id: Annotated[
        Optional[str],
        pydantic.Field(alias="merchantId"),
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = None
    r"""Filter products by merchant organization ID"""

    offset: Annotated[
        Optional[int],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = 0
    r"""Number of products to skip"""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["limit", "merchantId", "offset"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


class ListProductsResponseTypedDict(TypedDict):
    r"""List of products"""

    data: NotRequired[List[ProductTypedDict]]
    pagination: NotRequired[OffsetPaginationTypedDict]
    r"""Offset-based pagination response."""


class ListProductsResponse(BaseModel):
    r"""List of products"""

    data: Optional[List[Product]] = None

    pagination: Optional[OffsetPagination] = None
    r"""Offset-based pagination response."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["data", "pagination"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from .basesdk import BaseSDK
from paygentic_sdk import errors, models, utils
from paygentic_sdk._hooks import HookContext
from paygentic_sdk.types import OptionalNullable, UNSET
from paygentic_sdk.utils import get_security_from_env
from paygentic_sdk.utils.unmarshal_json_response import unmarshal_json_response
from typing import Any, List, Mapping, Optional, Union


class EntitlementsV0(BaseSDK):
    def list_active(
        self,
        *,
        customer_id: str,
        product_id: Optional[str] = None,
        subscription_id: Optional[str] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.GetActiveEntitlementsResponse:
        r"""List by Customer

        :param customer_id: The customer ID to get active entitlements for.
        :param product_id: The product ID to get active entitlements for.
        :param subscription_id: The subscription ID to get active entitlements for.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.GetActiveEntitlementsRequest(
            customer_id=customer_id,
            product_id=product_id,
            subscription_id=subscription_id,
        )

        req = self._build_request(
            method="GET",
            path="/v0/entitlements/activeEntitlements",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="getActiveEntitlements",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["400", "403", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(
                models.GetActiveEntitlementsResponse, http_res
            )
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(errors.BadRequestUnion, http_res)
            raise errors.BadRequest(response_data, http_res)
        if utils.match_response(http_res, "403", "application/json"):
            response_data = unmarshal_json_response(errors.ErrorData, http_res)
            raise errors.Error(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(errors.ErrorData, http_res)
            raise errors.Error(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.PaygenticDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.PaygenticDefaultError(
                "API error occurred", http_res, http_res_text
            )

        raise errors.PaygenticDefaultError("Unexpected response received", http_res)

    async def list_active_async(
        self,
        *,
        customer_id: str,
        product_id: Optional[str] = None,
        subscription_id: Optional[str] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.GetActiveEntitlementsResponse:
        r"""List by Customer

        :param customer_id: The customer ID to get active entitlements for.
        :param product_id: The product ID to get active entitlements for.
        :param subscription_id: The subscription ID to get active entitlements for.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.GetActiveEntitlementsRequest(
            customer_id=customer_id,
            product_id=product_id,
            subscription_id=subscription_id,
        )

        req = self._build_request_async(
            method="GET",
            path="/v0/entitlements/activeEntitlements",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="getActiveEntitlements",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["400", "403", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(
                models.GetActiveEntitlementsResponse, http_res
            )
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(errors.BadRequestUnion, http_res)
            raise errors.BadRequest(response_data, http_res)
        if utils.match_response(http_res, "403", "application/json"):
            response_data = unmarshal_json_response(errors.ErrorData, http_res)
            raise errors.Error(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(errors.ErrorData, http_res)
            raise errors.Error(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.PaygenticDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.PaygenticDefaultError(
                "API error occurred", http_res, http_res_text
            )

        raise errors.PaygenticDefaultError("Unexpected response received", http_res)

    def create(
        self,
        *,
        customer_id: str,
        entitlement_data: Union[
            List[models.EntitlementDatum], List[models.EntitlementDatumTypedDict]
        ],
        merchant_id: str,
        max_uses: Optional[int] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.Entitlement:
        r"""Create

        Create an entitlement for a customer. This temporarily reserves funds in the customer's wallet to guarantee payment for future usage.

        :param customer_id: Unique identifier for a customer
        :param entitlement_data: Array of consumption metrics with quantities to pre-authorize payment for.
        :param merchant_id: Unique identifier for a merchant/organization
        :param max_uses: Maximum consumption events allowed before expiration (optional). Defaults to 1 for global entitlements, 100 for regional entitlements.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.CreateEntitlementRequest(
            customer_id=customer_id,
            entitlement_data=utils.get_pydantic_model(
                entitlement_data, List[models.EntitlementDatum]
            ),
            max_uses=max_uses,
            merchant_id=merchant_id,
        )

        req = self._build_request(
            method="POST",
            path="/v0/entitlements",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request, False, False, "json", models.CreateEntitlementRequest
            ),
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="createEntitlement",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["400", "403", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "201", "application/json"):
            return unmarshal_json_response(models.Entitlement, http_res)
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(errors.BadRequestUnion, http_res)
            raise errors.BadRequest(response_data, http_res)
        if utils.match_response(http_res, "403", "application/json"):
            response_data = unmarshal_json_response(errors.ErrorData, http_res)
            raise errors.Error(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(errors.ErrorData, http_res)
            raise errors.Error(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.PaygenticDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.PaygenticDefaultError(
                "API error occurred", http_res, http_res_text
            )

        raise errors.PaygenticDefaultError("Unexpected response received", http_res)

    async def create_async(
        self,
        *,
        customer_id: str,
        entitlement_data: Union[
            List[models.EntitlementDatum], List[models.EntitlementDatumTypedDict]
        ],
        merchant_id: str,
        max_uses: Optional[int] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.Entitlement:
        r"""Create

        Create an entitlement for a customer. This temporarily reserves funds in the customer's wallet to guarantee payment for future usage.

        :param customer_id: Unique identifier for a customer
        :param entitlement_data: Array of consumption metrics with quantities to pre-authorize payment for.
        :param merchant_id: Unique identifier for a merchant/organization
        :param max_uses: Maximum consumption events allowed before expiration (optional). Defaults to 1 for global entitlements, 100 for regional entitlements.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.CreateEntitlementRequest(
            customer_id=customer_id,
            entitlement_data=utils.get_pydantic_model(
                entitlement_data, List[models.EntitlementDatum]
            ),
            max_uses=max_uses,
            merchant_id=merchant_id,
        )

        req = self._build_request_async(
            method="POST",
            path="/v0/entitlements",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request, False, False, "json", models.CreateEntitlementRequest
            ),
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="createEntitlement",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["400", "403", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "201", "application/json"):
            return unmarshal_json_response(models.Entitlement, http_res)
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(errors.BadRequestUnion, http_res)
            raise errors.BadRequest(response_data, http_res)
        if utils.match_response(http_res, "403", "application/json"):
            response_data = unmarshal_json_response(errors.ErrorData, http_res)
            raise errors.Error(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(errors.ErrorData, http_res)
            raise errors.Error(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.PaygenticDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.PaygenticDefaultError(
                "API error occurred", http_res, http_res_text
            )

        raise errors.PaygenticDefaultError("Unexpected response received", http_res)



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from datetime import datetime
from paygentic_sdk.types import BaseModel, UNSET_SENTINEL, UnrecognizedStr
import pydantic
from pydantic import model_serializer
from typing import Any, Dict, Literal, Optional, Union
from typing_extensions import Annotated, NotRequired, TypedDict


SourceObject = Literal["source",]


ProcessingMode = Union[
    Literal[
        "automatic",
        "manual",
    ],
    UnrecognizedStr,
]


SourceType = Literal["stripe_revenue",]


class SourceTypedDict(TypedDict):
    id: str
    r"""Unique identifier for a source"""
    object: SourceObject
    created_at: datetime
    enabled: bool
    merchant_id: str
    r"""Unique identifier for an organization"""
    name: str
    plan_id: str
    r"""Unique identifier for a plan"""
    processing_mode: ProcessingMode
    type: SourceType
    updated_at: datetime
    config: NotRequired[Dict[str, Any]]
    description: NotRequired[str]
    metadata: NotRequired[Dict[str, Any]]


class Source(BaseModel):
    id: str
    r"""Unique identifier for a source"""

    object: SourceObject

    created_at: Annotated[datetime, pydantic.Field(alias="createdAt")]

    enabled: bool

    merchant_id: Annotated[str, pydantic.Field(alias="merchantId")]
    r"""Unique identifier for an organization"""

    name: str

    plan_id: Annotated[str, pydantic.Field(alias="planId")]
    r"""Unique identifier for a plan"""

    processing_mode: Annotated[ProcessingMode, pydantic.Field(alias="processingMode")]

    type: SourceType

    updated_at: Annotated[datetime, pydantic.Field(alias="updatedAt")]

    config: Optional[Dict[str, Any]] = None

    description: Optional[str] = None

    metadata: Optional[Dict[str, Any]] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["config", "description", "metadata"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


try:
    Source.model_rebuild()
except NameError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from paygentic_sdk.types import BaseModel, UnrecognizedStr
import pydantic
from typing import Literal, Union
from typing_extensions import Annotated, TypeAliasType, TypedDict


Operator = Union[
    Literal[
        "equals",
        "contains",
        "between",
        "greaterThan",
        "lessThan",
        "domain",
    ],
    UnrecognizedStr,
]
r"""The comparison operator to use"""


RuleConditionType = Union[
    Literal[
        "date",
        "customerName",
        "customerEmail",
        "amount",
    ],
    UnrecognizedStr,
]
r"""The type of field to evaluate"""


FromTypedDict = TypeAliasType("FromTypedDict", Union[str, float])


From = TypeAliasType("From", Union[str, float])


ToTypedDict = TypeAliasType("ToTypedDict", Union[str, float])


To = TypeAliasType("To", Union[str, float])


class ValueTypedDict(TypedDict):
    from_: FromTypedDict
    to: ToTypedDict


class Value(BaseModel):
    from_: Annotated[From, pydantic.Field(alias="from")]

    to: To


ValueUnionTypedDict = TypeAliasType(
    "ValueUnionTypedDict", Union[ValueTypedDict, str, float]
)
r"""The value to compare against"""


ValueUnion = TypeAliasType("ValueUnion", Union[Value, str, float])
r"""The value to compare against"""


class RuleConditionTypedDict(TypedDict):
    r"""A single condition that must be met for a rule to trigger auto-acceptance. Multiple conditions in a rule use AND logic."""

    operator: Operator
    r"""The comparison operator to use"""
    type: RuleConditionType
    r"""The type of field to evaluate"""
    value: ValueUnionTypedDict
    r"""The value to compare against"""


class RuleCondition(BaseModel):
    r"""A single condition that must be met for a rule to trigger auto-acceptance. Multiple conditions in a rule use AND logic."""

    operator: Operator
    r"""The comparison operator to use"""

    type: RuleConditionType
    r"""The type of field to evaluate"""

    value: ValueUnion
    r"""The value to compare against"""


try:
    Value.model_rebuild()
except NameError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from datetime import datetime
from paygentic_sdk.types import BaseModel, UNSET_SENTINEL
import pydantic
from pydantic import model_serializer
from typing import List, Literal, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


ActiveEntitlementObject = Literal["entitlement",]


class ActiveEntitlementTypedDict(TypedDict):
    id: NotRequired[str]
    r"""Unique identifier for an entitlement"""
    object: NotRequired[ActiveEntitlementObject]
    billable_metrics: NotRequired[List[str]]
    r"""List of billable metrics that this entitlement covers."""
    customer_id: NotRequired[str]
    r"""Unique identifier for a customer"""
    plan_id: NotRequired[str]
    r"""Unique identifier for a plan"""
    product_id: NotRequired[str]
    r"""Unique identifier for a product"""
    subscription_id: NotRequired[str]
    r"""Unique identifier for a subscription"""
    verified_at: NotRequired[datetime]
    r"""Timestamp when the entitlement was checked."""


class ActiveEntitlement(BaseModel):
    id: Optional[str] = None
    r"""Unique identifier for an entitlement"""

    object: Optional[ActiveEntitlementObject] = "entitlement"

    billable_metrics: Annotated[
        Optional[List[str]], pydantic.Field(alias="billableMetrics")
    ] = None
    r"""List of billable metrics that this entitlement covers."""

    customer_id: Annotated[Optional[str], pydantic.Field(alias="customerId")] = None
    r"""Unique identifier for a customer"""

    plan_id: Annotated[Optional[str], pydantic.Field(alias="planId")] = None
    r"""Unique identifier for a plan"""

    product_id: Annotated[Optional[str], pydantic.Field(alias="productId")] = None
    r"""Unique identifier for a product"""

    subscription_id: Annotated[
        Optional[str], pydantic.Field(alias="subscriptionId")
    ] = None
    r"""Unique identifier for a subscription"""

    verified_at: Annotated[Optional[datetime], pydantic.Field(alias="verifiedAt")] = (
        None
    )
    r"""Timestamp when the entitlement was checked."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(
            [
                "id",
                "object",
                "billableMetrics",
                "customerId",
                "planId",
                "productId",
                "subscriptionId",
                "verifiedAt",
            ]
        )
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


try:
    ActiveEntitlement.model_rebuild()
except NameError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from paygentic_sdk.types import BaseModel, UNSET_SENTINEL
from paygentic_sdk.utils import FieldMetadata, PathParamMetadata, RequestMetadata
from pydantic import model_serializer
from typing import Dict, Literal, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


UpdateFeatureType = Literal[
    "metered",
    "static",
    "boolean",
]
r"""Updated feature type"""


class UpdateFeatureRequestBodyTypedDict(TypedDict):
    key: NotRequired[str]
    r"""Updated feature key slug. Sample values: 'increased-api-limit', 'new-storage-tier'"""
    name: NotRequired[str]
    r"""Updated feature name."""
    type: NotRequired[UpdateFeatureType]
    r"""Updated feature type"""
    metadata: NotRequired[Dict[str, str]]
    r"""Updated metadata for the feature"""


class UpdateFeatureRequestBody(BaseModel):
    key: Optional[str] = None
    r"""Updated feature key slug. Sample values: 'increased-api-limit', 'new-storage-tier'"""

    name: Optional[str] = None
    r"""Updated feature name."""

    type: Optional[UpdateFeatureType] = None
    r"""Updated feature type"""

    metadata: Optional[Dict[str, str]] = None
    r"""Updated metadata for the feature"""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["key", "name", "type", "metadata"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


class UpdateFeatureRequestTypedDict(TypedDict):
    id: str
    r"""The unique identifier of the feature"""
    request_body: UpdateFeatureRequestBodyTypedDict


class UpdateFeatureRequest(BaseModel):
    id: Annotated[
        str, FieldMetadata(path=PathParamMetadata(style="simple", explode=False))
    ]
    r"""The unique identifier of the feature"""

    request_body: Annotated[
        UpdateFeatureRequestBody,
        FieldMetadata(request=RequestMetadata(media_type="application/json")),
    ]



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from paygentic_sdk.types import BaseModel, UNSET_SENTINEL
import pydantic
from pydantic import model_serializer
from typing import Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class AddressTypedDict(TypedDict):
    line1: NotRequired[str]
    line2: NotRequired[str]
    city: NotRequired[str]
    state: NotRequired[str]
    country: NotRequired[str]
    r"""2 character ISO 3166 country code"""
    zip_code: NotRequired[str]


class Address(BaseModel):
    line1: Optional[str] = None

    line2: Optional[str] = None

    city: Optional[str] = None

    state: Optional[str] = None

    country: Optional[str] = None
    r"""2 character ISO 3166 country code"""

    zip_code: Annotated[Optional[str], pydantic.Field(alias="zipCode")] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["line1", "line2", "city", "state", "country", "zipCode"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


try:
    Address.model_rebuild()
except NameError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from paygentic_sdk.types import BaseModel, UNSET_SENTINEL
from paygentic_sdk.utils import FieldMetadata, PathParamMetadata, RequestMetadata
import pydantic
from pydantic import model_serializer
from typing import List, Literal, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


UpdatePlanTaxBehavior = Literal[
    "exclusive",
    "inclusive",
]
r"""Whether tax is added on top of the price (exclusive) or included in the price (inclusive)"""


class UpdatePlanRequestBodyTypedDict(TypedDict):
    billing_interval: NotRequired[str]
    r"""Recurring billing period frequency. Sample values: 'monthly' for monthly billing, 'yearly' for annual billing, 'weekly' for weekly billing"""
    default_tax_code: NotRequired[str]
    r"""Default tax code for plan line items. Common values: 'eservice' (electronically supplied services), 'saas' (software as a service), 'consulting', 'ebook', 'standard', 'reduced', 'exempt'. Full list available via GET /tax/codes endpoint."""
    default_tax_rate: NotRequired[float]
    r"""Fallback tax rate (as percentage) if automatic tax calculation is unavailable"""
    description: NotRequired[str]
    r"""Plan details explaining included features and limits. Sample values: 'Claude API access with 500K tokens monthly allowance', 'Unlimited cloud storage plus real-time analytics tools', 'Complete machine learning infrastructure with GPU access', 'Flexible usage-based pricing with no monthly commitment'"""
    invoice_display_name: NotRequired[str]
    r"""Plan name shown on billing statements. Sample values: 'LLM API Basic Plan', 'Data Warehouse Business', 'ML Platform Enterprise', 'Pay-Per-Use Model'"""
    name: NotRequired[str]
    r"""Plan identifier visible to customers. Sample values: 'Basic Tier', 'Business Package', 'Enterprise Solution', 'Metered Billing', 'Free Tier', 'Premium Access'"""
    prices: NotRequired[List[str]]
    r"""Array of price IDs to associate with this plan"""
    tax_behavior: NotRequired[UpdatePlanTaxBehavior]
    r"""Whether tax is added on top of the price (exclusive) or included in the price (inclusive)"""
    renewal_reminder_enabled: NotRequired[bool]
    r"""Whether to send renewal reminder emails to customers before their subscription renews"""
    renewal_reminder_days: NotRequired[int]
    r"""Number of days before renewal to send the reminder email"""


class UpdatePlanRequestBody(BaseModel):
    billing_interval: Annotated[
        Optional[str], pydantic.Field(alias="billingInterval")
    ] = None
    r"""Recurring billing period frequency. Sample values: 'monthly' for monthly billing, 'yearly' for annual billing, 'weekly' for weekly billing"""

    default_tax_code: Annotated[
        Optional[str], pydantic.Field(alias="defaultTaxCode")
    ] = None
    r"""Default tax code for plan line items. Common values: 'eservice' (electronically supplied services), 'saas' (software as a service), 'consulting', 'ebook', 'standard', 'reduced', 'exempt'. Full list available via GET /tax/codes endpoint."""

    default_tax_rate: Annotated[
        Optional[float], pydantic.Field(alias="defaultTaxRate")
    ] = None
    r"""Fallback tax rate (as percentage) if automatic tax calculation is unavailable"""

    description: Optional[str] = None
    r"""Plan details explaining included features and limits. Sample values: 'Claude API access with 500K tokens monthly allowance', 'Unlimited cloud storage plus real-time analytics tools', 'Complete machine learning infrastructure with GPU access', 'Flexible usage-based pricing with no monthly commitment'"""

    invoice_display_name: Annotated[
        Optional[str], pydantic.Field(alias="invoiceDisplayName")
    ] = None
    r"""Plan name shown on billing statements. Sample values: 'LLM API Basic Plan', 'Data Warehouse Business', 'ML Platform Enterprise', 'Pay-Per-Use Model'"""

    name: Optional[str] = None
    r"""Plan identifier visible to customers. Sample values: 'Basic Tier', 'Business Package', 'Enterprise Solution', 'Metered Billing', 'Free Tier', 'Premium Access'"""

    prices: Optional[List[str]] = None
    r"""Array of price IDs to associate with this plan"""

    tax_behavior: Annotated[
        Optional[UpdatePlanTaxBehavior], pydantic.Field(alias="taxBehavior")
    ] = None
    r"""Whether tax is added on top of the price (exclusive) or included in the price (inclusive)"""

    renewal_reminder_enabled: Annotated[
        Optional[bool], pydantic.Field(alias="renewalReminderEnabled")
    ] = None
    r"""Whether to send renewal reminder emails to customers before their subscription renews"""

    renewal_reminder_days: Annotated[
        Optional[int], pydantic.Field(alias="renewalReminderDays")
    ] = None
    r"""Number of days before renewal to send the reminder email"""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(
            [
                "billingInterval",
                "defaultTaxCode",
                "defaultTaxRate",
                "description",
                "invoiceDisplayName",
                "name",
                "prices",
                "taxBehavior",
                "renewalReminderEnabled",
                "renewalReminderDays",
            ]
        )
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


class UpdatePlanRequestTypedDict(TypedDict):
    id: str
    r"""The unique identifier of the plan"""
    request_body: UpdatePlanRequestBodyTypedDict


class UpdatePlanRequest(BaseModel):
    id: Annotated[
        str, FieldMetadata(path=PathParamMetadata(style="simple", explode=False))
    ]
    r"""The unique identifier of the plan"""

    request_body: Annotated[
        UpdatePlanRequestBody,
        FieldMetadata(request=RequestMetadata(media_type="application/json")),
    ]


try:
    UpdatePlanRequestBody.model_rebuild()
except NameError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from paygentic_sdk.types import BaseModel, UNSET_SENTINEL, UnrecognizedStr
import pydantic
from pydantic import model_serializer
from typing import Literal, Optional, Union
from typing_extensions import Annotated, NotRequired, TypedDict


FeePriceModel = Literal["standard",]
r"""The pricing model. Fees only support standard pricing."""


FeePricePaymentTerm = Union[
    Literal[
        "in_advance",
        "in_arrears",
    ],
    UnrecognizedStr,
]
r"""When the fee is charged relative to the billing period."""


class PropertiesTypedDict(TypedDict):
    unit_price: str
    r"""The unit price in dollars (e.g., '10.00')."""


class Properties(BaseModel):
    unit_price: Annotated[str, pydantic.Field(alias="unitPrice")]
    r"""The unit price in dollars (e.g., '10.00')."""


class FeePriceTypedDict(TypedDict):
    r"""The price configuration for a fee within a subscription context."""

    id: str
    r"""The unique identifier of the price."""
    fee_id: str
    r"""The unique identifier of the fee."""
    model: FeePriceModel
    r"""The pricing model. Fees only support standard pricing."""
    payment_term: FeePricePaymentTerm
    r"""When the fee is charged relative to the billing period."""
    invoice_display_name: str
    r"""The name to display on invoices for this fee."""
    properties: PropertiesTypedDict
    tax_rate: NotRequired[float]
    r"""The tax rate as a percentage (e.g., 10 for 10%)."""


class FeePrice(BaseModel):
    r"""The price configuration for a fee within a subscription context."""

    id: str
    r"""The unique identifier of the price."""

    fee_id: Annotated[str, pydantic.Field(alias="feeId")]
    r"""The unique identifier of the fee."""

    model: FeePriceModel
    r"""The pricing model. Fees only support standard pricing."""

    payment_term: Annotated[FeePricePaymentTerm, pydantic.Field(alias="paymentTerm")]
    r"""When the fee is charged relative to the billing period."""

    invoice_display_name: Annotated[str, pydantic.Field(alias="invoiceDisplayName")]
    r"""The name to display on invoices for this fee."""

    properties: Properties

    tax_rate: Annotated[Optional[float], pydantic.Field(alias="taxRate")] = None
    r"""The tax rate as a percentage (e.g., 10 for 10%)."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["taxRate"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


try:
    Properties.model_rebuild()
except NameError:
    pass
try:
    FeePrice.model_rebuild()
except NameError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from paygentic_sdk.types import BaseModel, UNSET_SENTINEL
from pydantic import model_serializer
from typing import Optional
from typing_extensions import NotRequired, TypedDict


class OffsetPaginationTypedDict(TypedDict):
    r"""Offset-based pagination response."""

    limit: NotRequired[int]
    r"""Number of items returned in the current page."""
    offset: NotRequired[int]
    r"""Number of items skipped."""
    total: NotRequired[int]
    r"""Total number of items available."""


class OffsetPagination(BaseModel):
    r"""Offset-based pagination response."""

    limit: Optional[int] = None
    r"""Number of items returned in the current page."""

    offset: Optional[int] = None
    r"""Number of items skipped."""

    total: Optional[int] = None
    r"""Total number of items available."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["limit", "offset", "total"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .entitlementtemplate import EntitlementTemplate, EntitlementTemplateTypedDict
from datetime import datetime
from paygentic_sdk.types import (
    BaseModel,
    Nullable,
    OptionalNullable,
    UNSET,
    UNSET_SENTINEL,
)
import pydantic
from pydantic import model_serializer
from typing import Dict, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class IssueEntitlementRequestTypedDict(TypedDict):
    customer_id: str
    r"""Unique identifier for a customer"""
    feature_id: str
    r"""The feature to grant access to."""
    template: EntitlementTemplateTypedDict
    r"""Template for the entitlement. Boolean for simple on/off features, static for features with configuration, metered for usage-based features."""
    active_from: NotRequired[datetime]
    r"""When the entitlement becomes active. Defaults to now."""
    active_to: NotRequired[Nullable[datetime]]
    r"""When the entitlement expires. Null means no expiration."""
    subscription_id: NotRequired[Nullable[str]]
    r"""Optional subscription ID to associate with this entitlement."""
    metadata: NotRequired[Dict[str, str]]
    r"""Additional metadata for the entitlement."""


class IssueEntitlementRequest(BaseModel):
    customer_id: Annotated[str, pydantic.Field(alias="customerId")]
    r"""Unique identifier for a customer"""

    feature_id: Annotated[str, pydantic.Field(alias="featureId")]
    r"""The feature to grant access to."""

    template: EntitlementTemplate
    r"""Template for the entitlement. Boolean for simple on/off features, static for features with configuration, metered for usage-based features."""

    active_from: Annotated[Optional[datetime], pydantic.Field(alias="activeFrom")] = (
        None
    )
    r"""When the entitlement becomes active. Defaults to now."""

    active_to: Annotated[
        OptionalNullable[datetime], pydantic.Field(alias="activeTo")
    ] = UNSET
    r"""When the entitlement expires. Null means no expiration."""

    subscription_id: Annotated[
        OptionalNullable[str], pydantic.Field(alias="subscriptionId")
    ] = UNSET
    r"""Optional subscription ID to associate with this entitlement."""

    metadata: Optional[Dict[str, str]] = None
    r"""Additional metadata for the entitlement."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["activeFrom", "activeTo", "subscriptionId", "metadata"])
        nullable_fields = set(["activeTo", "subscriptionId"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            is_nullable_and_explicitly_set = (
                k in nullable_fields
                and (self.__pydantic_fields_set__.intersection({n}))  # pylint: disable=no-member
            )

            if val != UNSET_SENTINEL:
                if (
                    val is not None
                    or k not in optional_fields
                    or is_nullable_and_explicitly_set
                ):
                    m[k] = val

        return m


try:
    IssueEntitlementRequest.model_rebuild()
except NameError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from datetime import datetime
from paygentic_sdk.types import (
    BaseModel,
    Nullable,
    OptionalNullable,
    UNSET,
    UNSET_SENTINEL,
)
import pydantic
from pydantic import model_serializer
from typing_extensions import Annotated, NotRequired, TypedDict


class TestClockTypedDict(TypedDict):
    __test__ = False  # pyright: ignore[reportGeneralTypeIssues]

    id: str
    r"""Unique identifier for the test clock"""
    created_at: datetime
    r"""When the test clock was created"""
    current_time: datetime
    r"""The simulated current time of the test clock"""
    merchant_id: str
    r"""The merchant organization that owns this test clock"""
    updated_at: datetime
    r"""When the test clock was last updated"""
    deleted_at: NotRequired[Nullable[datetime]]
    r"""When the test clock was deleted (if applicable)"""
    description: NotRequired[Nullable[str]]
    r"""Description of the test clock's purpose"""
    name: NotRequired[Nullable[str]]
    r"""Name of the test clock"""


class TestClock(BaseModel):
    __test__ = False

    id: str
    r"""Unique identifier for the test clock"""

    created_at: Annotated[datetime, pydantic.Field(alias="createdAt")]
    r"""When the test clock was created"""

    current_time: Annotated[datetime, pydantic.Field(alias="currentTime")]
    r"""The simulated current time of the test clock"""

    merchant_id: Annotated[str, pydantic.Field(alias="merchantId")]
    r"""The merchant organization that owns this test clock"""

    updated_at: Annotated[datetime, pydantic.Field(alias="updatedAt")]
    r"""When the test clock was last updated"""

    deleted_at: Annotated[
        OptionalNullable[datetime], pydantic.Field(alias="deletedAt")
    ] = UNSET
    r"""When the test clock was deleted (if applicable)"""

    description: OptionalNullable[str] = UNSET
    r"""Description of the test clock's purpose"""

    name: OptionalNullable[str] = UNSET
    r"""Name of the test clock"""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["deletedAt", "description", "name"])
        nullable_fields = set(["deletedAt", "description", "name"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            is_nullable_and_explicitly_set = (
                k in nullable_fields
                and (self.__pydantic_fields_set__.intersection({n}))  # pylint: disable=no-member
            )

            if val != UNSET_SENTINEL:
                if (
                    val is not None
                    or k not in optional_fields
                    or is_nullable_and_explicitly_set
                ):
                    m[k] = val

        return m


try:
    TestClock.model_rebuild()
except NameError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .address import Address, AddressTypedDict
from .validtaxaddress import ValidTaxAddress, ValidTaxAddressTypedDict
from datetime import datetime
from paygentic_sdk.types import BaseModel, UNSET_SENTINEL
import pydantic
from pydantic import model_serializer
from typing import Dict, Literal, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


CustomerObject = Literal["customer",]


class CustomerOrganizationTypedDict(TypedDict):
    id: str
    name: str
    address: NotRequired[AddressTypedDict]
    billing_email: NotRequired[str]
    phone: NotRequired[str]


class CustomerOrganization(BaseModel):
    id: str

    name: str

    address: Optional[Address] = None

    billing_email: Annotated[Optional[str], pydantic.Field(alias="billingEmail")] = None

    phone: Optional[str] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["address", "billingEmail", "phone"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


class CustomerTypedDict(TypedDict):
    id: str
    r"""Unique identifier for a customer"""
    consumer_id: str
    r"""Unique identifier for an organization"""
    created_at: datetime
    merchant_id: str
    r"""Unique identifier for an organization"""
    updated_at: datetime
    valid_tax_address: ValidTaxAddressTypedDict
    r"""Indicates whether the consumer address is valid for tax calculation when using Paygentic Tax. If valid=false, tax calculation will be skipped and internal invoice flow with default tax rate will be used."""
    object: NotRequired[CustomerObject]
    organization: NotRequired[CustomerOrganizationTypedDict]
    tax_id: NotRequired[str]
    r"""Business tax registration identifier. Sample values: 'GB123456789' for UK VAT, 'DE123456789' for German VAT, 'FR12345678901' for French VAT. Enables inter-company tax handling and exemption from standard tax collection."""
    tax_rates: NotRequired[Dict[str, float]]
    r"""An object mapping plan IDs, metric IDs, or 'default' to a tax rate percentage (e.g., 13 for 13%)"""


class Customer(BaseModel):
    id: str
    r"""Unique identifier for a customer"""

    consumer_id: Annotated[str, pydantic.Field(alias="consumerId")]
    r"""Unique identifier for an organization"""

    created_at: Annotated[datetime, pydantic.Field(alias="createdAt")]

    merchant_id: Annotated[str, pydantic.Field(alias="merchantId")]
    r"""Unique identifier for an organization"""

    updated_at: Annotated[datetime, pydantic.Field(alias="updatedAt")]

    valid_tax_address: Annotated[
        ValidTaxAddress, pydantic.Field(alias="validTaxAddress")
    ]
    r"""Indicates whether the consumer address is valid for tax calculation when using Paygentic Tax. If valid=false, tax calculation will be skipped and internal invoice flow with default tax rate will be used."""

    object: Optional[CustomerObject] = "customer"

    organization: Optional[CustomerOrganization] = None

    tax_id: Annotated[Optional[str], pydantic.Field(alias="taxId")] = None
    r"""Business tax registration identifier. Sample values: 'GB123456789' for UK VAT, 'DE123456789' for German VAT, 'FR12345678901' for French VAT. Enables inter-company tax handling and exemption from standard tax collection."""

    tax_rates: Annotated[
        Optional[Dict[str, float]], pydantic.Field(alias="taxRates")
    ] = None
    r"""An object mapping plan IDs, metric IDs, or 'default' to a tax rate percentage (e.g., 13 for 13%)"""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["object", "organization", "taxId", "taxRates"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


try:
    CustomerOrganization.model_rebuild()
except NameError:
    pass
try:
    Customer.model_rebuild()
except NameError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from paygentic_sdk.types import BaseModel, UNSET_SENTINEL
from paygentic_sdk.utils import FieldMetadata, PathParamMetadata, RequestMetadata
from pydantic import model_serializer
from typing import Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class UpdateFeeRequestBodyTypedDict(TypedDict):
    description: NotRequired[str]
    r"""Revised explanation of what the fee represents."""
    name: NotRequired[str]
    r"""Updated label for the fee."""


class UpdateFeeRequestBody(BaseModel):
    description: Optional[str] = None
    r"""Revised explanation of what the fee represents."""

    name: Optional[str] = None
    r"""Updated label for the fee."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["description", "name"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


class UpdateFeeRequestTypedDict(TypedDict):
    id: str
    r"""The unique identifier of the fee"""
    request_body: UpdateFeeRequestBodyTypedDict


class UpdateFeeRequest(BaseModel):
    id: Annotated[
        str, FieldMetadata(path=PathParamMetadata(style="simple", explode=False))
    ]
    r"""The unique identifier of the fee"""

    request_body: Annotated[
        UpdateFeeRequestBody,
        FieldMetadata(request=RequestMetadata(media_type="application/json")),
    ]



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from paygentic_sdk.types import BaseModel, UNSET_SENTINEL
from pydantic import model_serializer
from typing import Optional
from typing_extensions import NotRequired, TypedDict


class ValidTaxAddressTypedDict(TypedDict):
    r"""Indicates whether the consumer address is valid for tax calculation when using Paygentic Tax. If valid=false, tax calculation will be skipped and internal invoice flow with default tax rate will be used."""

    valid: bool
    r"""Whether the consumer address is valid for tax calculation when using Paygentic Tax"""
    message: NotRequired[str]
    r"""Validation message if address is invalid. Sample values: 'Country is required for tax calculation', 'Invalid postal code format', 'State/province required for this country'"""


class ValidTaxAddress(BaseModel):
    r"""Indicates whether the consumer address is valid for tax calculation when using Paygentic Tax. If valid=false, tax calculation will be skipped and internal invoice flow with default tax rate will be used."""

    valid: bool
    r"""Whether the consumer address is valid for tax calculation when using Paygentic Tax"""

    message: Optional[str] = None
    r"""Validation message if address is invalid. Sample values: 'Country is required for tax calculation', 'Invalid postal code format', 'State/province required for this country'"""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["message"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from dataclasses import dataclass, field
import httpx
from paygentic_sdk.errors import PaygenticError
from paygentic_sdk.models import deletecustomerop as models_deletecustomerop
from paygentic_sdk.types import BaseModel
from typing import Optional


class DeleteCustomerConflictErrorData(BaseModel):
    error: Optional[models_deletecustomerop.DeleteCustomerError] = None
    message: Optional[str] = None
    details: Optional[models_deletecustomerop.DeleteCustomerDetails] = None


@dataclass(unsafe_hash=True)
class DeleteCustomerConflictError(PaygenticError):
    r"""Customer cannot be deleted due to active dependencies"""

    data: DeleteCustomerConflictErrorData = field(hash=False)

    def __init__(
        self,
        data: DeleteCustomerConflictErrorData,
        raw_response: httpx.Response,
        body: Optional[str] = None,
    ):
        fallback = body or raw_response.text
        message = str(data.message) or fallback
        super().__init__(message, raw_response, body)
        object.__setattr__(self, "data", data)



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from datetime import datetime
from paygentic_sdk.types import BaseModel
from paygentic_sdk.utils import FieldMetadata, PathParamMetadata, RequestMetadata
import pydantic
from typing import Union
from typing_extensions import Annotated, TypeAliasType, TypedDict


class RequestBody2TypedDict(TypedDict):
    advance_by: str
    r"""ISO 8601 duration format indicating clock advancement distance. Sample values: 'P1M' moves forward one month, 'P7D' moves forward seven days, 'PT2H' moves forward two hours, 'P1Y2M3DT4H5M6S' moves forward one year, two months, three days, four hours, five minutes, and six seconds"""


class RequestBody2(BaseModel):
    advance_by: Annotated[str, pydantic.Field(alias="advanceBy")]
    r"""ISO 8601 duration format indicating clock advancement distance. Sample values: 'P1M' moves forward one month, 'P7D' moves forward seven days, 'PT2H' moves forward two hours, 'P1Y2M3DT4H5M6S' moves forward one year, two months, three days, four hours, five minutes, and six seconds"""


class RequestBody1TypedDict(TypedDict):
    current_time: datetime
    r"""New absolute time for the test clock (must be forward in time)"""


class RequestBody1(BaseModel):
    current_time: Annotated[datetime, pydantic.Field(alias="currentTime")]
    r"""New absolute time for the test clock (must be forward in time)"""


AdvanceTestClockRequestBodyTypedDict = TypeAliasType(
    "AdvanceTestClockRequestBodyTypedDict",
    Union[RequestBody1TypedDict, RequestBody2TypedDict],
)


AdvanceTestClockRequestBody = TypeAliasType(
    "AdvanceTestClockRequestBody", Union[RequestBody1, RequestBody2]
)


class AdvanceTestClockRequestTypedDict(TypedDict):
    id: str
    r"""Test clock ID"""
    request_body: AdvanceTestClockRequestBodyTypedDict


class AdvanceTestClockRequest(BaseModel):
    id: Annotated[
        str, FieldMetadata(path=PathParamMetadata(style="simple", explode=False))
    ]
    r"""Test clock ID"""

    request_body: Annotated[
        AdvanceTestClockRequestBody,
        FieldMetadata(request=RequestMetadata(media_type="application/json")),
    ]


try:
    RequestBody2.model_rebuild()
except NameError:
    pass
try:
    RequestBody1.model_rebuild()
except NameError:
    pass
